/*
beri_pisi2.h		some functions to send numbers to MSP430 uart


    Copyright (C) 2022 Marko Cebokli   S57UUU
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
    See the GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program. If not, see <https://www.gnu.org/licenses/>. 

*/


//---------------------------------------------------
//verzija na interrupt - da ni busy wait
//da char v circ buffer. Ce je tx1flg idle, da prvi char direktno v TX regisster
int poslji_byte(char ch)
{
ntx1 = txwp1-txrp1;
if (ntx1 < 0) ntx1 += TXB1SZ;
if (ntx1 == TXB1SZ-1) return -1;	//buffer full
if (tx1flg == 1)	//TX idle
  {
  UCA0TXBUF=ch;
  tx1flg=0;
  }
else
  {
  txbuf1[txwp1++]=ch;
  if (txwp1 >= TXB1SZ) txwp1 -= TXB1SZ; 
  }
return 0;
}

//*************************************
//od tu dalje ni vec hardware dependent
//*************************************

//-------------------------------------------------
void poslji_crlf()
{
poslji_byte(13); 
poslji_byte(10); 
}

//---------------------------------------------------
void poslji_string(char *s)
{
int i=0;

while ((s[i]!=0)&&(i<256))
  {
  poslji_byte(s[i]);  
  i++;
  }
}

//---------------------------------------------------
//binary string can contain zeros
void poslji_bstring(char *s, int nc)
{
int i;

for (i=0;i<nc;i++)  poslji_byte(s[i]);  
}

//------------------------------------------------------------------
//pretvorba 16bit unsigned stevilke v string in posiljanje
//vedno poslje vseh pet stevilk (n.pr. 00067)
void poslji32dec(unsigned long adval, int crlf)
{
unsigned char ch;  

ch=0x30;
while (adval>=1000000000)
  {
  adval-=1000000000;
  ch++;
  }
poslji_byte(ch);  
ch=0x30;
while (adval>=100000000)
  {
  adval-=100000000;
  ch++;
  }
poslji_byte(ch);  
ch=0x30;
while (adval>=10000000)
  {
  adval-=10000000;
  ch++;
  }
poslji_byte(ch);  
ch=0x30;
while (adval>=1000000)
  {
  adval-=1000000;
  ch++;
  }
poslji_byte(ch);  
ch=0x30;
while (adval>=100000)
  {
  adval-=100000;
  ch++;
  }
poslji_byte(ch);  
ch=0x30;
while (adval>=10000)
  {
  adval-=10000;
  ch++;
  }
poslji_byte(ch);  
ch=0x30;
while (adval>=1000)
  {
  adval-=1000;
  ch++;
  }
poslji_byte(ch);  
ch=0x30;
while (adval>=100)
  {
  adval-=100;
  ch++;
  }
poslji_byte(ch);  
ch=0x30;
while (adval>=10)
  {
  adval-=10;
  ch++;
  }
poslji_byte(ch);  
ch=0x30;
while (adval>=1)
  {
  adval-=1;
  ch++;
  }
poslji_byte(ch);  
  
if (crlf==0) return;

poslji_byte(13);  
poslji_byte(10);  

}

//------------------------------------------------------------------
//pretvorba 32bit signed stevilke v string in posiljanje
//vedno poslje vseh deset stevilk (n.pr. 00067)
void poslji32Sdec(long adval, int crlf)
{
unsigned char ch;  

if (adval<0)
  {
  ch='-';
  adval=-adval;
  }
else
  ch=' ';
poslji_byte(ch);  

poslji32dec((unsigned long)adval,crlf);

}


//------------------------------------------------------------------
//pretvorba 16bit unsigned stevilke v string in posiljanje
//vedno poslje vseh pet stevilk (n.pr. 00067)
void poslji16dec(unsigned int adval, int crlf)
{
unsigned char ch;  
  
ch=0x30;
while (adval>=10000)
  {
  adval-=10000;
  ch++;
  }
poslji_byte(ch);  
ch=0x30;
while (adval>=1000)
  {
  adval-=1000;
  ch++;
  }
poslji_byte(ch);  
ch=0x30;
while (adval>=100)
  {
  adval-=100;
  ch++;
  }
poslji_byte(ch);  
ch=0x30;
while (adval>=10)
  {
  adval-=10;
  ch++;
  }
poslji_byte(ch);  
ch=0x30;
while (adval>=1)
  {
  adval-=1;
  ch++;
  }
poslji_byte(ch);  
  
if (crlf==0) return;

poslji_byte(13);  
poslji_byte(10);  

}

//------------------------------------------------------------------
//pretvorba 16bit unsigned stevilke v string in posiljanje
//vedno poslje vseh pet stevilk (n.pr. 00067)
//z decimalno piko na mestu p  [1...4]
void poslji16dec_p(unsigned int adval, int crlf, int p)
{
unsigned char ch;  
  
ch=0x30;
while (adval>=10000)
  {
  adval-=10000;
  ch++;
  }
poslji_byte(ch);
if (p==4) poslji_string(".");
ch=0x30;
while (adval>=1000)
  {
  adval-=1000;
  ch++;
  }
poslji_byte(ch);  
if (p==3) poslji_string(".");
ch=0x30;
while (adval>=100)
  {
  adval-=100;
  ch++;
  }
poslji_byte(ch);  
if (p==2) poslji_string(".");
ch=0x30;
while (adval>=10)
  {
  adval-=10;
  ch++;
  }
poslji_byte(ch);  
if (p==1) poslji_string(".");
ch=0x30;
while (adval>=1)
  {
  adval-=1;
  ch++;
  }
poslji_byte(ch);  
  
if (crlf==0) return;

poslji_byte(13);  
poslji_byte(10);  

}

//------------------------------------------------------------------
//pretvorba 16bit signed stevilke v string in posiljanje
//vedno poslje vseh pet stevilk (n.pr. 00067)
void poslji16Sdec(int adval, int crlf)
{
unsigned char ch;  

if (adval<0)
  {
  ch='-';
  adval=-adval;
  }
else
  ch=' ';
poslji_byte(ch);  

poslji16dec((unsigned int)adval,crlf);

}

//------------------------------------------------------------------
//pretvorba 16bit unsigned stevilke v string in posiljanje
//vedno poslje vseh pet stevilk (n.pr. 00067)
void poslji16hex(unsigned char adval, int crlf)
{
unsigned char ch;  
  
ch=0x30;
while (adval>=4096)
  {
  adval-=16;
  ch++;
  }
if (ch>57) ch += 7;
poslji_byte(ch);  
ch=0x30;
ch=0x30;
while (adval>=256)
  {
  adval-=1;
  ch++;
  }
if (ch>57) ch += 7;
poslji_byte(ch);  
ch=0x30;
while (adval>=16)
  {
  adval-=16;
  ch++;
  }
if (ch>57) ch += 7;
poslji_byte(ch);  
ch=0x30;
ch=0x30;
while (adval>=1)
  {
  adval-=1;
  ch++;
  }
if (ch>57) ch += 7;
poslji_byte(ch);  
  
if (crlf==0) return;

poslji_byte(13);  
poslji_byte(10);  

}

//------------------------------------------------------------------
//pretvorba 16bit unsigned stevilke v string in posiljanje
//vedno poslje vseh pet stevilk (n.pr. 00067)
void poslji8dec(unsigned char adval, int crlf)
{
unsigned char ch;  
  
ch=0x30;
while (adval>=100)
  {
  adval-=100;
  ch++;
  }
poslji_byte(ch);  
ch=0x30;
while (adval>=10)
  {
  adval-=10;
  ch++;
  }
poslji_byte(ch);  
ch=0x30;
while (adval>=1)
  {
  adval-=1;
  ch++;
  }
poslji_byte(ch);  
  
if (crlf==0) return;

poslji_byte(13);  
poslji_byte(10);  

}

//------------------------------------------------------------------
//pretvorba 16bit unsigned stevilke v string in posiljanje
//vedno poslje vseh pet stevilk (n.pr. 00067)
void poslji8hex(unsigned char adval, int crlf)
{
unsigned char ch;  
  
ch=0x30;
while (adval>=16)
  {
  adval-=16;
  ch++;
  }
if (ch>57) ch += 7;
poslji_byte(ch);  
ch=0x30;
while (adval>=1)
  {
  adval-=1;
  ch++;
  }
if (ch>57) ch += 7;
poslji_byte(ch);  
  
if (crlf==0) return;

poslji_byte(13);  
poslji_byte(10);  

}

//---------------------------------------------
//dvomestno decimalno stevilo brez predznaka da v char
unsigned char txt2uchar2(char *txt)
{
unsigned char ch;

ch=0;
if ((txt[0]>=48)&&(txt[0]<=57)) ch+=10*(txt[0]-48);
if ((txt[1]>=48)&&(txt[1]<=57)) ch+=txt[1]-48;
return ch;
}

//---------------------------------------------
//dvomestno decimalno stevilo s predznakom da v char
char txt2char2(char *txt)
{
char ch;

ch=0;
if ((txt[1]>=48)&&(txt[1]<=57)) ch+=10*(txt[1]-48);
if ((txt[2]>=48)&&(txt[2]<=57)) ch+=txt[2]-48;
if (txt[0] == '-') ch*=-1;
return ch;
}

//---------------------------------------------
//petmestno decimalno stevilo brez predznaka da v int
unsigned int txt2uint5(char *txt)
{
unsigned int i;

i=0;
if ((txt[0]>=48)&&(txt[0]<=57)) i+=10000*(txt[1]-48);
if ((txt[1]>=48)&&(txt[1]<=57)) i+=1000*(txt[2]-48);
if ((txt[2]>=48)&&(txt[2]<=57)) i+=100*(txt[3]-48);
if ((txt[3]>=48)&&(txt[3]<=57)) i+=10*(txt[4]-48);
if ((txt[4]>=48)&&(txt[4]<=57)) i+=txt[1]-48;
return i;
}

