/*
mbsd.c  Message Based Self Disciplining GPSDO    version 0.1


    Copyright (C) 2022 Marko Cebokli   S57UUU
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
    See the GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program. If not, see <https://www.gnu.org/licenses/>. 

compile:
msp430-gcc -Os -Wall -g -mmcu=msp430g2553 mbsd.c -o mbsd.elf
program MCU:
mspdebug rf2500 "prog mbsd.elf"

Pin  3	RXD	from ublox
Pin  4	TXD	to ublox
Pin 10	P2.2	SW debug out
Pin 13	TA1.2	Coarse PWM
Pin 14	TA0.1	Fine PWM

*/

#include  <msp430.h>
#include <in430.h>
#include <inttypes.h>


#define TXB1SZ 128

//globalne spremenljivke
int bst;	//stevec bytov na I2S izhodu
char dacout[8]={0,0,0,0,0,0,0,0};
unsigned long ldac,rdac;
char txbuf1[TXB1SZ];
int txrp1,txwp1;	//read in write kazalca v txbuf1
int tx1flg;		//flag tx idle (postavi ga irq funkcija, ko poslje zadnjega)
int ntx1;		//koliko char je v txbuf1

int32_t qerr,oldqerr;
int32_t uqerr;		//unwrapped
int32_t clkd;
uint32_t pwm;

#include "beri_pisi2.h"
#include "ubxdec.h"

//--------------------------------------------------------------
void setup_ublox()
{
  
set_ubx_protocol();    //NMEA ukaz
__delay_cycles(3200000);

set_cfg_msg(1,0x4,1);		//nav dop
__delay_cycles(3200000);
set_cfg_msg(1,0x22,1);		//nav clock
__delay_cycles(3200000);
set_cfg_msg(0xD,0x1,1);		//tim tp
__delay_cycles(3200000);
  
}

//--------------------------------------------------------
void discipline()
{
int32_t dqerr;
int32_t maxqerr=18491;	//assuming 2*max qerr = 0.5/26MHz [ps]
maxqerr=20850;		//for dqerr rollover fixing (empirical value)

/*
//HP10811, -0.3Hz/V  minimum qerr deviation (cca 60 RMS)	"fast loop"
int32_t clk_p=-1000000;	//clkd loop, proportional
int32_t qer_p=-19000;	//qerr loop, proportional	-5000...-20000?
int32_t qer_d=-200000;	//qerr loop, differential	-90000...-320000?
*/
/*
//HP10811, -0.3Hz/V  trade off 					"medium loop"
int32_t clk_p=-1000000;	//clkd loop, proportional
int32_t qer_p=-2;	//qerr loop, proportional	-2
int32_t qer_d=-415;	//qerr loop, differential	-415
*/

//HP10811, -0.3Hz/V  good stability compared to rubidium	"slow loop"
int32_t clk_p=-1000000;	//clkd loop, proportional
int32_t qer_p=-0;	//qerr loop, proportional	 -0
int32_t qer_d=-32;	//qerr loop, differential	-32


dqerr = qerr-oldqerr;
oldqerr=qerr;
if (dqerr > (maxqerr>>1)) dqerr-=maxqerr;
else if (dqerr < -(maxqerr>>1)) dqerr+=maxqerr;
uqerr += dqerr;

//    PWM:  1000000 = 1.5mHz
if (clkd !=0)	//coarse loop, clkd based
  {
  pwm -= clkd*clk_p;
//  uqerr=0;	//prevent buildup at high frequency offsets
  uqerr=qerr;	//prevent buildup at high frequency offsets
  }
else		//fine loop, qerr based
  {
  pwm -= (uqerr*qer_p + dqerr*qer_d)/32;
  }

/*  
//TEST
static int cnt;
if (cnt>200) cnt=0;  
cnt++;
//if (cnt<100) pwm=380890000; else pwm=380925000;	//1000000 = 1.6mHz @ 10MHz (HP)
if (cnt<100) pwm=380000000; else pwm=381760000;		//1mHz = 590000
*/ 
  
  
TA1CCR2 = (uint32_t)pwm>>16;			//coarse PWM
TA0CCR1 = ((uint32_t)pwm & 0x0000FC00) >> 10;	//fine PWM

poslji_string(" clkd= ");poslji16Sdec((int16_t)clkd,0);
poslji_string("  qerr= ");poslji16Sdec((int16_t)qerr,0);
poslji_string("  dqer= ");poslji16Sdec((int16_t)dqerr,0);
poslji_string("  uqer= ");poslji32Sdec(uqerr,0);
poslji_string("  pwm= ");poslji32dec(pwm,1);
//poslji_string("  pwmc= ");poslji16dec((uint16_t)(pwm>>16),0);
//poslji_string("  pwmf= ");poslji8dec(((uint16_t)pwm & 0x0000FC00) >> 10,1);

//P2OUT ^= 0x04;	//debug izhod 
}

//***********************************************************
int main(void)
{

WDTCTL = WDTPW + WDTHOLD;       // Stop WDT
BCSCTL1 = CALBC1_16MHZ;         // Set DCO for SMCLK
DCOCTL = CALDCO_16MHZ;


P2DIR = 0x25;		//P2.0 = TA1.0 out  P2.2=SW debug out  P2.5 = PWM out
P2SEL |= 0x23;		//P2.0 = TA1.0 out  P2.1 = TA1.1 input  P2.5 = PWM out

TA0CTL = TASSEL_2 + MC_1;	//SMCLK, upmode
TA0CCR0 = 63;			//period  4us
TA0CCR1 = 31;			//PWM fine initial value
TA0CCTL1 = OUTMOD_7;		//reset/set, PWM

TA1CTL = TASSEL_2 + MC_1;	//SMCLK, upmode
TA1CCR0 = 9999;
TA1CCTL0 = OUTMOD_4; 		//toggle, 500Hz out
TA1CCR2 = 5000;			//PWM coarse
TA1CCTL2 = OUTMOD_7;		//reset/set, PWM

UCA0CTL1 |= UCSSEL_2;       // SMCLK
UCA0BR0 = 104;              // 16MHz 9600 bps
UCA0BR1 = 0;                // 16MHz 9600 bps
UCA0MCTL = UCBRS_0+UCBRF_3+UCOS16;       // Modulation UCBRSx = 1
UCA0CTL1 &= ~UCSWRST;       // Initialize USCI state machine
//IE2 |= UCA0RXIE;          // Enable USCI_A0 RX interrupt
IE2 |= UCA0TXIE;          // Enable USCI_A0 TX interrupt

P1SEL = BIT1 + BIT2 + BIT6; // P1.1 = RXD, P1.2 = TXD, P1.6 = TA0.1
P1SEL2 = BIT1 + BIT2;       // P1.1 = RXD, P1.2 = TXD
P1DIR |= 0x44;		    // P1.2 = TXD, P1.6 (TA0.1) = Fine PWM


static unsigned int stanje,dolzina,pkp;
uint8_t c;
uint8_t chka,chkb;
int chk;
int nc,nc2;   //stevca karakterjev za debug
char string[32];
int paklen;
uint8_t mclass,msgid;
uint8_t paket[64];
int i;
int dum;

txwp1=0; txrp1=0; tx1flg=1;

__enable_interrupt();

setup_ublox();

pwm=2278*65536;		//start value
pwm=5000*65536;		//start value
pwm=5100*65536;		//start value

stanje=0;
nc=0;
uqerr=0;
oldqerr=0;
while(1)
  {
  while (!(UC0IFG & 0x01)) dum=0;	//wait for next char
  c=UCA0RXBUF;
//  UCA0TXBUF = c; continue;
//  P2OUT ^= 0x04;	//debug izhod 
  nc++;
  switch (stanje)	//state machine for packet extraction
    {
    case 0:		//wait for 0xB5  (first sync byte)
      if (c==0xB5) stanje=1;
      break;
    case 1:		//wait for 0x62  (second sync byte)
      if (c==0x62) stanje=2; else stanje=0;
      break;
    case 2:		//mclass
      mclass = c;
      stanje=3;
      break;
    case 3:		//message ID
      msgid = c;
      stanje=5;
      break;
    case 5:		//length (LSB)
      dolzina = c;
      stanje=6;
      break;
    case 6:		//length (MSB)
      dolzina+=256*c;
      pkp=0;
      if (dolzina > 64)
        {
	poslji_string("Bad len= ");poslji16Sdec(dolzina,1);	//send some data to tty
        stanje=0;
	break;
        }
      paklen=dolzina;
      stanje=7;
      break;
    case 7:		//data bytes
      paket[pkp++]=c;
      dolzina--;
      if (dolzina==0)	stanje=8;
      break;
    case 8:		//check _A
      chka = c;
      stanje=9;
      break;
    case 9:		//check _B
//      poslji_string("Class= ");poslji8dec(mclass,0);
//      poslji_string(" msgid= ");poslji8dec(msgid,0);
//      poslji_string(" len= ");poslji16dec(paklen,1);
//      for (i=0;i<paklen;i++) {poslji8dec(paket[i],0);poslji_string(" ");}poslji_string("\r\n");
//      for (i=0;i<paklen;i++) {poslji8hex(paket[i],0);poslji_string(" ");}poslji_string("\r\n");
      chkb = c;
      pkp=0;
      chk=checksum(mclass, msgid, paklen, paket);
      if (chk != 256*chka+chkb)
        {
	poslji_string(" Bad chk= ");poslji16hex(chk,0);
	poslji_string(" ");poslji8hex(chka,0);
	poslji_string(" ");poslji8hex(chkb,1);
//	poslji_string("c");
        }
      else
        {
        obdelaj(mclass, msgid, paket, paklen);
//	poslji_string("OBD "); poslji16Sdec(paklen,1);
//	poslji_string("o");
	if (mclass==13) discipline();
        }
      nc=0;
      stanje=0;
      break;
    }
    
//  P2OUT ^= 0x04;	//debug out 
  }

return 0;
}

//////////////////////////////////////////////////////////////////////////
#pragma vector=USCIAB0TX_VECTOR
__interrupt void USCI0TX_ISR(void)
{
if (txrp1 != txwp1)
  {
  UCA0TXBUF=txbuf1[txrp1++];
  if (txrp1 >= TXB1SZ) txrp1 -= TXB1SZ; 
  }
else
  {
  tx1flg=1;
  UC0IFG &= ~UCA0TXIFG;
  }

//P2OUT ^= 0x04;	//debug out 
}
