/*
ubxdec.h     functions to decode some ublox protocol messages

    Copyright (C) 2022 Marko Cebokli   S57UUU
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
    See the GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program. If not, see <https://www.gnu.org/licenses/>. 

*/



//-------------------------------------------------------
unsigned int checksum(uint8_t mclass, uint8_t msgid, int len, uint8_t *paket)
{
uint8_t msg[64],cka,ckb;
int i;
unsigned int s;

msg[0] = mclass;
msg[1] = msgid;
msg[2] = len&0xFF;
s = len >> 8;
msg[3] = s;
for (i=0;i<len+1;i++) msg[i+4] = paket[i];

//printf("msg= ");for (i=0; i<len+4; i++) printf(" %02X",msg[i]); printf("\n");

cka=0; ckb=0;
for (i=0;i<len+4;i++)
  {
  cka += msg[i];  //cka &=0xFF;
  ckb += cka;     //ckb &= 0xFF;
  }
s=cka; s <<= 8; s += ckb;   //big endian (a je msB)
return s;
}

//-----------------------------------------------------------------
//switch to UBX protocol
int set_cfg_prt()
{
uint8_t buf[32];
uint16_t chk;
uint8_t portid=1;		//  1,2 = UARTs
uint8_t reserved0=0;
uint16_t txready=0;
uint32_t mode=1<<4 | 1<<6 | 1<<7 | 1<<11;
uint32_t baudrate=9600;
uint16_t inprotomask=3;		//2=NMEA  1=UBX
uint16_t outprotomask=1;
uint16_t reserved4=0;
uint16_t reserved5=0;

buf[0]=0xB5; buf[1]=0x62;	//header
buf[2]=0x06; buf[3]=0x00;	//class=CFG, ID=PRT
buf[4]=0x14; buf[5]=0x00;	//length

buf[6]=portid;
buf[7]=reserved0;
buf[8]=(uint16_t)txready&0xFF;
buf[9]=(uint16_t)txready>>8;
buf[10]=(uint32_t)mode&0xFF;
buf[11]=(uint32_t)mode>>8;
buf[12]=(uint32_t)mode>>16;
buf[13]=(uint32_t)mode>>24;
buf[14]=(uint32_t)baudrate&0xFF;
buf[15]=(uint32_t)baudrate>>8;
buf[16]=(uint32_t)baudrate>>16;
buf[17]=(uint32_t)baudrate>>24;
buf[18]=(uint16_t)inprotomask&0xFF;
buf[19]=(uint16_t)inprotomask>>8;
buf[20]=(uint16_t)outprotomask&0xFF;
buf[21]=(uint16_t)outprotomask>>8;
buf[22]=(uint16_t)reserved4&0xFF;
buf[23]=(uint16_t)reserved4>>8;
buf[24]=(uint16_t)reserved5&0xFF;
buf[25]=(uint16_t)reserved5>>8;

chk=checksum(0x06, 0x00, 20, &buf[6]);
//buf[26]=chk&0xFF;
//buf[27]=chk>>8;
buf[27]=chk&0xFF;
buf[26]=chk>>8;

//write(sport,buf,28);
//int i; printf("Sent: ");for (i=0; i<28; i++) printf(" %02X",buf[i]);printf("\n");
//decode_cfg_prt(&buf[6], 20);

poslji_bstring(buf,28);
//int i; poslji_string("prt: ");for (i=0; i<28; i++) {poslji8hex(buf[i],0);poslji_string(" ");}poslji_string("\r\n");

return 0;
}

//----------------------------------------------------------------
//NMEA command to switch on UBX protocol
int set_ubx_protocol()
{
//uint8_t buf[256],chs[8];
uint8_t chk;
//int port=1;
//int inproto=3;
//int outproto=1;
//int baudrate=9600;
//int autobaud=0;
int i;

uint8_t buf[64]="$PUBX,41,1,0003,0001,9600,0*";

//sprintf(buf,"$PUBX,41,%d,%04d,%04d,%d,%d*",port,inproto,outproto,baudrate,autobaud);
chk=0;
for (i=1;i<27;i++) chk^=buf[i];

buf[28]=0x30;
while (chk>=16)
  {
  chk-=16;
  buf[28]++;
  }
if (buf[28]>57) buf[28] += 7;

buf[29]=0x30;
while (chk>=1)
  {
  chk-=1;
  buf[29]++;
  }
if (buf[29]>57) buf[29] += 7;

buf[30]=13;
buf[31]=10;
buf[32]=0;

//sprintf(chs,"%02X",chk);
//chs[2]=13; chs[3]=10; chs[4]=0;
//strcat(buf,chs);
//printf("Sent: %s\n",buf);
  
//write(sport,buf,strlen(buf));
poslji_string(buf);poslji_string("\r\n");


return 0;
}

//----------------------------------------------------------------
//set rate for currnet target  (target=???? 0=I2C(DDS) 1=USART1 2=USART2 3=USB 4=SPI) 
int set_cfg_msg(uint8_t class, uint8_t msgid, uint8_t rate)
{
uint8_t buf[32];
uint16_t chk;

buf[0]=0xB5; buf[1]=0x62;	//header
buf[2]=0x06; buf[3]=0x01;	//class, ID
buf[4]=0x03; buf[5]=0x00;	//length

buf[6]=class;
buf[7]=msgid;
buf[8]=rate;

chk=checksum(0x06, 0x01, 3, &buf[6]);   //to dela  (dobim ACK)
buf[9]=(uint16_t)chk>>8;			//big end?  to dela!! (checksum da a v msB)
buf[10]=(uint16_t)chk&0xFF;

//write(sport,buf,11);
//int i; printf("Sent: ");for (i=0; i<11; i++) printf(" %02X",buf[i]);printf("\n");

poslji_bstring(buf,11);
//int i; poslji_string("msg: ");for (i=0; i<11; i++) {poslji8hex(buf[i],0);poslji_string(" ");}poslji_string("\r\n");
  
return 0; 
}

//---------------------------------------------------------
int set_cfg_reset(uint16_t navbbrmask, uint8_t resetmode)
{
uint8_t buf[32],reserved1;
uint16_t chk;

reserved1=0;

buf[0]=0xB5; buf[1]=0x62;	//header
buf[2]=0x06; buf[3]=0x04;	//class, ID
buf[4]=0x04; buf[5]=0x00;	//length

buf[6]=(uint16_t)navbbrmask & 0xFF;
buf[7]=(uint16_t)navbbrmask >> 8;
buf[8]=resetmode;
buf[9]=reserved1;
  
chk=checksum(0x06, 0x04, 4, &buf[6]);
buf[10]=(uint16_t)chk>>8;			//big end?
buf[11]=(uint16_t)chk&0xFF;

//write(sport,buf,12);
//int i; printf("Sent: ");for (i=0; i<12; i++) printf(" %02X",buf[i]);printf("\n");

poslji_bstring(buf,12);
//int i; poslji_string("res: ");for (i=0; i<12; i++) {poslji8hex(buf[i],0);poslji_string(" ");}poslji_string("\r\n");

return 0;
}

//----------------------------------------------------
int decode_nav_clock(unsigned char *paket, int paklen)
{
//uint32_t itow,tacc,facc;
//int32_t clkb,clkd;

//itow = paket[0] | (paket[1]<<8) | (paket[2]<<16) | (paket[3]<<24);
//clkb = paket[4] | (paket[5]<<8) | (paket[6]<<16) | (paket[7]<<24);
//if (clkb > (1<<31)) clkb -= 4294967296;
clkd = (long)paket[8] | ((long)paket[9]<<8) | ((long)paket[10]<<16) | ((long)paket[11]<<24);
if (clkd > (1<<31)) clkd -= 4294967296;
//tacc = paket[12] | (paket[13]<<8) | (paket[14]<<16) | (paket[15]<<24);
//facc = paket[16] | (paket[17]<<8) | (paket[14]<<18) | (paket[19]<<24);

//printf("NAV-CLOCK:   itow=%d  clkb=%d  clkd=%d  tacc=%d  facc=%d\n",itow,clkb,clkd,tacc,facc);
//poslji_string("clockd= "); poslji32Sdec(clkd,1);

return 0;
}

//----------------------------------------------------
int decode_nav_dop(unsigned char *paket, int paklen)
{
//uint32_t itow;
//uint16_t gdop,pdop,tdop,vdop,hdop,ndop,edop;

//itow = paket[0] | (paket[1]<<8) | (paket[2]<<16) | (paket[3]<<24);
//gdop = paket[4] | (paket[5]<<8);
//pdop = paket[6] | (paket[7]<<8);
//tdop = paket[8] | (paket[9]<<8);
//vdop = paket[10] | (paket[11]<<8);
//hdop = paket[12] | (paket[13]<<8);
//ndop = paket[14] | (paket[15]<<8);
//edop = paket[16] | (paket[17]<<8);

//printf("NAV-DOP:     itow=%d gdop=%d pdop=%d tdop=%d vdop=%d hdop=%d\n",itow,gdop,pdop,tdop,vdop,hdop);
//printf("             ndop=%d edop=%d\n",ndop,edop);
//poslji_string("NAV-DOP ");

return 0;
}

//----------------------------------------------------
int decode_tim_tp(unsigned char *paket, int paklen)
{
//uint32_t towms,towsubms;
//int32_t qerr;
//uint16_t week;
//uint8_t flags,reserved1;

//for (int i=0; i<paklen; i++) printf(" %02X",paket[i]); printf("\n");

//towms = paket[0] | (paket[1]<<8) | (paket[2]<<16) | (paket[3]<<24);
//towsubms = paket[4] | (paket[5]<<8) | (paket[6]<<16) | (paket[7]<<24);
qerr = (long)paket[8] | ((long)paket[9]<<8) | ((long)paket[10]<<16) | ((long)paket[11]<<24);
if (qerr > (1<<31)) qerr -= 4294967296;
//week = paket[12] | (paket[13]<<8);
//flags = paket[14];
//reserved1 = paket[15];
/*
printf("TIM-TP:      towms=%d towsubms=%d week=%d qerr=%d  ",towms,towsubms,week,qerr);
if (flags & 1)
  printf("UTC time\n");
else
  printf("GPS time\n");
*/
//poslji_string("qerr= ");poslji32Sdec(qerr,1);

return 0;
}

//-------------------------------------------------------------
//obdela en paket, ki ga je sprejel unpack()
void obdelaj(uint8_t mclass, uint8_t msgid, unsigned char *paket, int paklen)
{
int dec=0;
  
//printf("Obdelaj: mclass=0x%02X, ID=0x%02X, len=%d\n",mclass,msgid,paklen);

switch (mclass)
  {
  case 0x01:    //mclass NAV
    switch (msgid)
      {
      case 0x04:    //NAV-DOP
        decode_nav_dop(paket,paklen);
        dec=1;
        break;
      case 0x22:    //NAV-CLOCK
        decode_nav_clock(paket,paklen);
        dec=1;
        break;
      default:
//	printf("mclass NAV: unknown ID 0x%02X\n",msgid);
	poslji_string("NAV ID= ");poslji8dec(msgid,1);
	break;
      }
    break;
  case 0x05:    //mclass ACK
    switch (msgid)
      {
      case 0x01:    //
//        printf("ACK-ACK:     mclass=%d ID=%d\n",paket[0],paket[1]);  //zakaj izpise mclass=6
	poslji_string("ACk-ACK ");
        dec=1;		//zakaj izpisujem paket in ne mclass,msgid, oz kar konstante??
        break;
      case 0x00:    //
//        printf("ACK-NAK:     mclass=%d ID=%d ********** \n",paket[0],paket[1]);
	poslji_string("ACk-NAK ");
        dec=1;
        break;
      default:
//	printf("mclass ACK: unknown ID 0x%02X\n",msgid);
	poslji_string("ACK ID= ");poslji8dec(msgid,1);
	break;
      }
    break;
  case 0x0D:    //mclass TIM
    switch (msgid)
      {
      case 0x01:    //TIM-TP
        decode_tim_tp(paket,paklen);
        dec=1;
        break;
      case 0x03:    //TIM-TM2
        break;
      case 0x04:    //TIM-SVIN
        break;
      case 0x06:    //TM-VRFY
        break;
      default:
//	printf("mclass TIM: unknown ID 0x%02X\n",msgid);
	poslji_string("TIM ID= ");poslji8dec(msgid,1);
	break;
      }
    break;
  default:
//    printf("Unknown mclass 0x%02X\n",mclass);
    poslji_string("Class= ");poslji8dec(mclass,1);
    break;
  }
  
//if (dec==0) printf("Obdelaj: mclass=0x%02X, ID=0x%02X, len=%d NOT DECODED\n",mclass,msgid,paklen);
if (dec==0) poslji_string("Not dec \n");
}

